package controllers

import (
	"encoding/json"
	"hongze/hz_crm_api/models"
	"hongze/hz_crm_api/models/system"
	"hongze/hz_crm_api/utils"
	"strings"
	"time"
)

// 部门组
type SysGroupController struct {
	BaseAuthController
}

// @Title 新增分组
// @Description 新增分组接口
// @Param	request	body system.SysGroupAddReq true "type json string"
// @Success 200 新增成功
// @router /group/add [post]
func (this *SysGroupController) Add() {
	br := new(models.BaseResponse).Init()
	defer func() {
		this.Data["json"] = br
		this.ServeJSON()
	}()
	var req system.SysGroupAddReq
	err := json.Unmarshal(this.Ctx.Input.RequestBody, &req)
	if err != nil {
		br.Msg = "参数解析异常！"
		br.ErrMsg = "参数解析失败，Err:" + err.Error()
		return
	}
	if req.DepartmentId <= 0 {
		br.Msg = "请选择部门"
		br.ErrMsg = "部门ID不可为空"
		return
	}
	if req.GroupName == "" {
		br.Msg = "分组名称不能为空"
		return
	}
	groupNameArr := strings.Split(req.GroupName, ",")
	for _, v := range groupNameArr {
		count, err := system.GetSysGroupCount(req.DepartmentId, v)
		if err != nil {
			br.Msg = "获取数据失败"
			br.ErrMsg = "获取数据失败,Err:" + err.Error()
			return
		}
		if count <= 0 {
			item := new(system.SysGroup)
			item.GroupName = v
			item.DepartmentId = req.DepartmentId
			item.CreateTime = time.Now()
			groupId, e := system.AddSysGroup(item)
			if e != nil {
				br.Msg = "新增失败"
				br.ErrMsg = "新增失败，Err:" + e.Error()
				return
			}

			// 同步分组缓存
			var syncData system.SyncGroupData
			syncData.Source = utils.SOURCE_CRM_FLAG
			syncData.GroupId = int(groupId)
			_ = utils.Rc.LPush(utils.CACHE_SYNC_GROUP, syncData)
		}
	}

	br.Ret = 200
	br.Success = true
	br.Msg = "新增成功"
}

// @Title 修改分组
// @Description  修改分组接口
// @Param	request	body system.SysGroupEditReq true "type json string"
// @Success 200 修改成功
// @router /group/edit [post]
func (this *SysGroupController) Edit() {
	br := new(models.BaseResponse).Init()
	defer func() {
		this.Data["json"] = br
		this.ServeJSON()
	}()
	var req system.SysGroupEditReq
	err := json.Unmarshal(this.Ctx.Input.RequestBody, &req)
	if err != nil {
		br.Msg = "参数解析异常！"
		br.ErrMsg = "参数解析失败，Err:" + err.Error()
		return
	}
	if req.GroupId <= 0 {
		br.Msg = "参数错误"
		br.ErrMsg = "参数错误,GroupId 小于等于0 "
		return
	}
	if req.GroupName == "" {
		br.Msg = "分组名称不能为空"
		return
	}
	item, err := system.GetSysGroupByName(req.GroupName)
	if err != nil && err.Error() != utils.ErrNoRow() {
		br.Msg = "获取数据失败"
		br.ErrMsg = "获取数据失败,Err:" + err.Error()
		return
	}
	if item != nil {
		if item.GroupId != req.GroupId {
			br.Msg = "名称已存在，请重新输入"
			return
		}
	}
	err = system.ModifySysGroup(req.GroupName, req.GroupId)
	if err != nil {
		br.Msg = "修改失败"
		br.ErrMsg = "修改失败，Err:" + err.Error()
		return
	}

	// 同步分组缓存
	var syncData system.SyncGroupData
	syncData.Source = utils.SOURCE_CRM_FLAG
	syncData.GroupId = req.GroupId
	_ = utils.Rc.LPush(utils.CACHE_SYNC_GROUP, syncData)

	br.Ret = 200
	br.Success = true
	br.Msg = "修改成功"
}

// @Title 删除分组
// @Description 删除分组接口
// @Param	request	body system.SysGroupDeleteReq true "type json string"
// @Success 200 删除成功
// @router /group/delete [post]
func (this *SysGroupController) Delete() {
	br := new(models.BaseResponse).Init()
	defer func() {
		this.Data["json"] = br
		this.ServeJSON()
	}()
	var req system.SysGroupDeleteReq
	err := json.Unmarshal(this.Ctx.Input.RequestBody, &req)
	if err != nil {
		br.Msg = "参数解析异常！"
		br.ErrMsg = "参数解析失败，Err:" + err.Error()
		return
	}
	if req.GroupId <= 0 {
		br.Msg = "参数错误"
		br.ErrMsg = "参数错误,GroupId 小于等于0 "
		return
	}
	err = system.DeleteSysGroup(req.GroupId)
	if err != nil {
		br.Msg = "删除失败"
		br.ErrMsg = "删除失败，Err:" + err.Error()
		return
	}
	err = system.ClearSysUserGroup(req.GroupId)
	if err != nil {
		br.Msg = "删除失败"
		br.ErrMsg = "删除失败,情况分组失败，Err:" + err.Error()
		return
	}

	// 同步分组缓存
	var syncData system.SyncGroupData
	syncData.Source = utils.SOURCE_CRM_FLAG
	syncData.GroupId = req.GroupId
	_ = utils.Rc.LPush(utils.CACHE_SYNC_GROUP, syncData)

	br.Ret = 200
	br.Success = true
	br.Msg = "删除成功"
}

// SetSort
// @Title 分组排序
// @Description  分组排序
// @Param	request	body system.SysGroupEditReq true "type json string"
// @Success 200 修改成功
// @router /group/set_sort [post]
func (this *SysGroupController) SetSort() {
	br := new(models.BaseResponse).Init()
	defer func() {
		if br.ErrMsg == "" {
			br.IsSendEmail = false
		}
		this.Data["json"] = br
		this.ServeJSON()
	}()
	var req system.SysGroupSortReq
	err := json.Unmarshal(this.Ctx.Input.RequestBody, &req)
	if err != nil {
		br.Msg = "参数解析异常！"
		br.ErrMsg = "参数解析失败，Err:" + err.Error()
		return
	}

	// 部门排序
	if len(req.DepartmentIds) > 0 {
		updateArr := make([]*system.DepartmentSort, 0)
		for k, d := range req.DepartmentIds {
			updateArr = append(updateArr, &system.DepartmentSort{
				DepartmentId: d,
				Sort:         k + 1,
			})
		}
		if e := system.MultiUpdateDepartmentSort(updateArr); e != nil {
			br.Msg = "操作失败"
			br.ErrMsg = "批量更新部门排序失败, Err: " + e.Error()
			return
		}
	}

	// 分组排序/小组排序
	if len(req.GroupIds) > 0 || len(req.TeamIds) > 0 {
		updateArr := make([]*system.GroupSort, 0)
		if len(req.GroupIds) > 0 {
			for k, g := range req.GroupIds {
				updateArr = append(updateArr, &system.GroupSort{
					GroupId: g,
					Sort:    k + 1,
				})
			}
		}
		if len(req.TeamIds) > 0 {
			for k, t := range req.TeamIds {
				updateArr = append(updateArr, &system.GroupSort{
					GroupId: t,
					Sort:    k + 1,
				})
			}
		}

		if len(updateArr) > 0 {
			if e := system.MultiUpdateGroupSort(updateArr); e != nil {
				br.Msg = "操作失败"
				br.ErrMsg = "批量更新分组排序失败, Err: " + e.Error()
				return
			}

			// 同步分组缓存
			for _, g := range updateArr {
				var syncData system.SyncGroupData
				syncData.Source = utils.SOURCE_CRM_FLAG
				syncData.GroupId = g.GroupId
				_ = utils.Rc.LPush(utils.CACHE_SYNC_GROUP, syncData)
			}
		}
	}

	br.Ret = 200
	br.Success = true
	br.Msg = "操作成功"
}
